/*
 * Copyright 2003 Amber Archer Consulting Co., Inc. All rights reserved.
 * AMBER ARCHER PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package org.amberarcher.xml.tool.highlight;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.net.URL;
import java.net.MalformedURLException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Result;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Class that encapsulates the transformation from marked-up XHTML to plain XHTML
 * with mark-up representing highlighted source.
 *
 * @author <a href="mailto:kdowney@amberarcher.com">Kyle F. Downey (Amber Archer)</a>
 * @version Revision$
 */
public class HighlightEngine implements HighlightContext {

    public void highlight(InputSource src, Result res) throws HighlightException {
        try {
            // for the HighlightContext
            this.currentSysId = src.getSystemId();

            // parse the input
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            dbf.setNamespaceAware(true);
            DocumentBuilder docBuilder = dbf.newDocumentBuilder();

            Node inputNode = docBuilder.parse(src);

            // replace all highlight nodes with mark-up
            highlightNode(inputNode);

            // output the transformed result
            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer outputTransform = transformerFactory.newTransformer();
            outputTransform.transform(new DOMSource(inputNode), res);
        } catch (ParserConfigurationException e) {
            throw new HighlightException(e);
        } catch (TransformerConfigurationException e) {
            throw new HighlightException(e);
        } catch (TransformerException e) {
            throw new HighlightException(e);
        } catch (SAXException e) {
            throw new HighlightException(e);
        } catch (IOException e) {
            throw new HighlightException(e);
        }
    }

    public void addHighlighter(String nsUri, String nodeName, Highlighter h) {
        highlighters.put(nsUri + "#" + nodeName, h);
        h.setHighlightContext(this);
    }

    public URL resolveRelativeUri(String uri) throws MalformedURLException {
        return new URL(new URL(currentSysId), uri);
    }

    private void highlightNode(Node node) throws HighlightException {
        NodeList childNodes = node.getChildNodes();
        for (int ii = 0; ii < childNodes.getLength(); ii++) {
            highlightNode(childNodes.item(ii));
        }

        String nodeName = node.getLocalName();
        String ns = node.getNamespaceURI();
        String fqNodeName = ns + "#" + nodeName;

        Highlighter h = getHighlighter(fqNodeName);
        if (null != h) {
            Node resNode = h.highlight(node);

            Node parentNode = node.getParentNode();
            resNode = node.getOwnerDocument().importNode(resNode, true);
            parentNode = node.getParentNode();

            parentNode.removeChild(node);
            parentNode.appendChild(resNode);
        }
    }

    private Highlighter getHighlighter(String fqNodeName) {
        return (Highlighter) highlighters.get(fqNodeName);
    }

    private Map highlighters = new HashMap();
    private String currentSysId;
}
