/*
 * Copyright 2003 Amber Archer Consulting Co., Inc. All rights reserved.
 * AMBER ARCHER PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package org.amberarcher.xml.tool.highlight;

import java.io.FileOutputStream;
import java.io.OutputStream;
import javax.xml.transform.stream.StreamResult;
import org.xml.sax.InputSource;

/**
 * Entry point for running the highlighter. Usage:
 *
 * <pre>
 *   java org.amberarcher.xml.tool.highlight.Main input.xhtml output.xhtml
 * </pre>
 *
 * @author <a href="mailto:kdowney@amberarcher.com">Kyle F. Downey</a>
 * @version $Revision: 1.3 $
 */
public class Main {
    public static final void main(String[] args) throws Exception {
        if (args.length == 0) {
            System.out.println("Usage:");
            System.out.println("\tjava org.amberarcher.xml.tool.highlight.Main input.xhtml output.xhtml");
            System.exit(1);
        }

        String inputFileName = args[0];
        OutputStream outputStream = null;
        if (args.length < 2) {
            outputStream = System.out;
        } else {
            outputStream = new FileOutputStream(args[1]);
        }

        HighlightEngine he = new HighlightEngine();

        he.addHighlighter("http://schemas.amberarcher.org/xmltools/highlight.html", "source-code",
                new SourceHighlighter());
        he.addHighlighter("http://schemas.amberarcher.org/xmltools/highlight.html", "xml",
                new XmlHighlighter());

        he.highlight(new InputSource(inputFileName), new StreamResult(outputStream));
    }

}
