/*
 * Copyright 2003 Amber Archer Consulting Co., Inc. All rights reserved.
 * AMBER ARCHER PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package org.amberarcher.xml.tool.highlight;

import java.io.InputStream;
import java.net.URL;
import java.net.MalformedURLException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamSource;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Highlighter that takes <code>&lt;highlight:xml&gt;</code> tag content
 * and converts it to highlighted XML.
 *
 * @author <a href="mailto:kdowney@amberarcher.com">Kyle F. Downey</a>
 * @version $Revision: 1.3 $
 */
public class XmlHighlighter implements Highlighter {
    public XmlHighlighter() {
        try {
            TransformerFactory transformerFactory = TransformerFactory.newInstance();

            InputStream xslIn = XmlHighlighter.class.getResourceAsStream("/etc/stylesheet/xml2xhtml.xsl");
            Source xslSrc = new StreamSource(xslIn);
            this.stylesheet = transformerFactory.newTransformer(xslSrc);
            this.docBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
        } catch (Exception e) {
            throw new RuntimeException("XML configuration error", e);
        }
    }

    public Node highlight(Node replaceNode) throws HighlightException {
        Source src = new DOMSource(getSourceDocument(replaceNode));
        DOMResult res = new DOMResult();

        try {
            stylesheet.transform(src, res);
        } catch (Exception e) {
            throw new HighlightException("stylesheet error", e);
        }

        return res.getNode().getFirstChild();
    }

    public void setHighlightContext(HighlightContext ctx) {
        this.ctx = ctx;
    }

    private Document getSourceDocument(Node replaceNode) throws HighlightException {
        Element replaceElem = (Element) replaceNode;
        String srcUriTxt = replaceElem.getAttribute("href");
        if ((null != srcUriTxt) && (srcUriTxt.trim().length() > 0)) {
            URL srcUri = null;
            try {
                srcUri = new URL(srcUriTxt);
            } catch (MalformedURLException e) {
                try {
                    srcUri = ctx.resolveRelativeUri(srcUriTxt);
                } catch (MalformedURLException ex) {
                    throw new HighlightException("could not resolve relative URI '" + srcUriTxt + "'");
                }
            }

            try {
                return docBuilder.parse(srcUri.toExternalForm());
            } catch (Exception e) {
                throw new HighlightException("bad href: " + srcUriTxt, e);
            }
        } else {
            Element childElem = null;

            NodeList nodes = replaceNode.getChildNodes();
            for (int ii = 0; ii < nodes.getLength(); ii++) {
                Node node = nodes.item(ii);
                if (node instanceof Element) {
                    childElem = (Element) node;
                    break;
                }
            }

            if (null == childElem) {
                throw new HighlightException("no element content found");
            }

            Document embeddedDoc = docBuilder.newDocument();
            childElem = (Element) embeddedDoc.importNode(childElem, true);
            embeddedDoc.appendChild(childElem);

            return embeddedDoc;
        }
    }


    private DocumentBuilder docBuilder;
    private Transformer stylesheet;
    private HighlightContext ctx;
}
